/*************************************************************************
* An object that is used to check one or more feeds:
* + Require to be updated
* + Download them if required
* + Parse them if required and generate a local HTML copy
* + See if feed has changed and indicate it apropriately
* Got many ideas for this from sage.mozdev.org's updatechecker.js
*************************************************************************/
const UTF8 = "UTF-8";
var hxUpdateFeed = {
    checking: false,
    httpReq: null,

    feedList: null,
    downloadList: null,
    nameList: null,
    urlList: null,
    localPathList: null,
    localHTMLPathList: null,
    descriptionList: null,
    lastModifiedList: null,

    currentFeed: null,
    currentName: null,
    currentURL: null,
    currentPath: null,
    currentHTMLPath: null,
    currentDescription: null,
    newLastModified: null,

    isClick: null,

    // A method to initiate the checking
    begin: function(folderId, aClick) {
	dump("hxUpdateFeed.begin(): Entered with " + folderId + "\n");
	// Don't start a new check if one already running
	if (this.checking) return;

	this.isClick = aClick;
	feedList = new Array();
	this.downloadList = new Array();
	this.nameList = new Array();
	this.urlList = new Array();
	this.localPathList = new Array();
	this.localHTMLPathList = new Array();
	this.descriptionList = new Array();
	this.lastModifiedList = new Array();

	// Get a list of all the feeds that are contained in the folder
	if (this.isClick) {
	    feedList.push(folderId);
	} else {
	    feedList = getBMDSCChildren(folderId);
	}

	// Tun on flag indicating we are busy
	this.checking = true;

	// Go through each feed one by one and see if:
	// a. It has been enough time since LastVisitDate
	// b. If there is a local copy of the feed
	// c. If there is a parsed copy of the feed
	for (var i = 0; i < feedList.length; i++) {
	    var aInput, aArcNameURI, aArcURLURI, aArcDescriptionURI, aArcLastVisitDateURI;

	    aInput = (typeof(feedList[i]) == "string") ? RDF.GetResource(feedList[i]) : feedList[i];
	    aArcNameURI = RDF.GetResource(NC_NS + "Name");
	    aArcURLURI = RDF.GetResource(NC_NS + "URL");
	    aArcDescriptionURI = RDF.GetResource(NC_NS + "Description");
	    aArcLastVisitDateURI = RDF.GetResource(WEB_NS + "LastVisitDate");

	    // Get the name of the feed
	    var name = getBMDSProperty(aInput, aArcNameURI);
	    var description = getBMDSProperty(aInput, aArcDescriptionURI);
	    var feedURL = getBMDSProperty(aInput, aArcURLURI);
	    var lastVisited = getBMDSProperty(aInput, aArcLastVisitDateURI);
	    lastVisited /= 1000;	// nsIRDFDate is in microseconds JS Date() deals with milliseconds
	    dump("begin(): LastVisitDate according to BMDS is " + lastVisited + "\n");
	    var localPath = absPathToLocal(feedURL);
	    var localHTMLPath = localPath + ".html";

	    var lastUpdatedAt = new Date(lastVisited);
	    var currentTime = new Date();
	    dump("hxUpdateFeed.begin(): The time is " + currentTime + "\n");

	    // 1000 milliseconds in every second
	    // 60 seconds in every minute
	    // user wants to update every updateInterval minutes
	    var oneHour = 1000 * 60 * updateInterval;

	    var interval = currentTime - lastUpdatedAt;
	    dump("hxUpdateFeed.begin(): interval since last update is " + interval + " ms\n");

	    if (interval > oneHour) {
		// It has been updateInterval since last update. This needs updating.
		dump("hxUpdateFeed.begin(): Feed " + name + " is stale.  Scheduling for update.\n");
		this.downloadList.push(feedList[i]);
		this.nameList.push(name);
		this.urlList.push(feedURL);
		this.localPathList.push(localPath);
		this.localHTMLPathList.push(localHTMLPath);
		this.descriptionList.push(description);
	    } else {
		dump("hxUpdateFeed.begin(): Feed " + name + " is fresh.  Let us see if we have a local copy of feed.\n");

		try {
		    var localFile = new File(localPath);
		    if (localFile.exists() && localFile.size > 0) {
			dump("hxUpdateFeed.begin(): Local copy of feed " + name + " exists.\n");
		    } else {
			dump("hxUpdateFeed.begin(): Local copy of feed " + name + " does not exist. Scheduling for update.\n");
			this.downloadList.push(feedList[i]);
			this.nameList.push(name);
			this.urlList.push(feedURL);
			this.localPathList.push(localPath);
			this.localHTMLPathList.push(localHTMLPath);
			this.descriptionList.push(description);
			continue;
		    }

		    var localHTMLFile = new File(localHTMLPath);
		    if (localHTMLFile.exists() && localHTMLFile.size > 0) {
			dump("hxUpdateFeed.begin(): Parsed copy of feed " + name + " exists.\n");
		    } else {
			dump("hxUpdateFeed.begin(): Parsed copy of feed " + name + " does not exist. Scheduling for update.\n");

			// TODO We must only parse. No need to re-download
			this.downloadList.push(feedList[i]);
			this.nameList.push(name);
			this.urlList.push(feedURL);
			this.localPathList.push(localPath);
			this.localHTMLPathList.push(localHTMLPath);
			this.descriptionList.push(description);
			continue;
		    }
		} catch (e) {
		    dump("hxUpdateFeed.begin(): Error trying to see if feed copy parsed and present. Ignoring this one.\n");
		    continue;
		}
	    }

	    // localPath = absPathToLocal(fetchURL);
	}
	if (this.downloadList.length > 0) {
	    this.downloadNext();
	} else {
	    // Nothing to download. Already have a copy
	    if (hxUpdateFeed.isClick) {
		// Was this in response to a user click?
		dump("Responding to user click Opening " + localHTMLPath + "\n");
		openNewsInTab(localHTMLPath);

		var arcSource = (typeof(folderId) == "string") ? RDF.GetResource(folderId) : folderId;
		var arcDescription = RDF.GetResource(NC_NS + "Description");
		var node = BMDS.GetTarget(arcSource, arcDescription, true);

		var curDesc;
		if (node) {
		    try {
			curDesc = node.QueryInterface(kRDFLITIID).Value;
		    } catch (e) {
			curDesc = "hxnew -- 1";
		    }
		} else {
		    curDesc = "hxnew -- 1";
		}

		var curStatus = curDesc.split(" -- ")[0];
		var curLastUp = curDesc.split(" -- ")[1];
		var newStatus = "hxold";
		var newDesc = newStatus + " -- " + curLastUp;
		var newNode = RDF.GetLiteral(newDesc);

		if (node) {
		    try {
			BMDS.Change(arcSource, arcDescription, node, newNode);
			BMDS.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
		    } catch (e) {
			dump("hxUpdateFeed.checkForNew(): Error changing description of bookmark: " + hxUpdateFeed.currentURL + " " + e + "\n");
		    }
		} else {
		    dump("hxUpdateFeed.checkForNew(): No descrition existing. Creating one.\n");
		    // A description does not exist. Create it
		    try {
			BMDS.Assert(arcSource, arcDescription, newNode, true);
			BMDS.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
		    } catch (e) {
			dump("hxUpdateFeed.checkForNew(): Error asserting description of bookmark: " + hxUpdateFeed.currentURL + " " + e + "\n");
		    }
		}
	    }
	    this.done();
	}
    },

    // A method to stop the checking
    done: function() {
	if (this.checking) {
	    try {
		this.httpReq.abort();
	    } catch (e) {}
	}
	this.checking = false;
    },

    // A method to go through the feeds that need checking one by one 
    // and download
    downloadNext: function() {
	dump("hxUpdateFeed.downloadNext(): Entered.\n");
	this.currentFeed = this.downloadList.shift();
	this.currentName = this.nameList.shift();
	this.currentURL = this.urlList.shift();
	this.currentPath = this.localPathList.shift();
	this.currentHTMLPath = this.localHTMLPathList.shift();
	this.currentDescription = this.descriptionList.shift();
	dump("hxUpdateFeed.downloadNext(): Going to download from " + this.currentURL + ".\n");

	if (! this.currentURL) {
	    // Not really necessary because we took care of this in begin()
	    this.checkForNew(false,0);
	}

	if (this.httpReq) {
	    // If there is an open request, abort it.
	    dump("hxUpdateFeed.downloadNext(): There seems to be an existing request. Aborting it.\n");
	    this.httpReq.abort();
	}

	dump("hxUpdateFeed.downloadNext(): Initializing request object to download " + this.currentURL + " to " + this.currentPath + ".\n");
	this.httpReq = new XMLHttpRequest();
	this.httpReq.parent = this;
	this.httpReq.open("GET", this.currentURL);

	// Event handlers
	this.httpReq.onload = this.httpLoaded;
	this.httpReq.onerror = this.httpError;
	this.httpReq.onreadystatechange = this.httpReadyStateChange;

	dump("hxUpdateFeed.downloadNext(): Initiating request object to download " + this.currentURL + ".\n");
	try {
	    var userAgent = navigator.appCodeName + "/" + navigator.appVersion + " " + strbundle.GetStringFromName("readerlabel") + " " + strbundle.GetStringFromName("readerversion") + "(" + strbundle.GetStringFromName("readerurl") + ")";
	    if (prefAlterUA) {
		this.httpReq.setRequestHeader("User-Agent", userAgent);
	    }
	    this.httpReq.setRequestHeader("Accept", "*/*");
	    this.httpReq.overrideMimeType("text/xml");
	    this.httpReq.send(null);
	    this.onCheck(this.currentName, this.currentURL);
	} catch (e) {
	    dump("hxUpdateFeed.downloadNext(): There was a problem sending the request. Aborting it : " + e + "\n");
	    this.httpReq.abort();
	    this.checkForNew(false,0);
	}
    },

    httpError: function(e) {
	dump("hxUpdateFeed.httpError(): There was a problem sending the request. Aborting it : " + e + "\n");
	hxUpdateFeed.httpReq.abort();
	hxUpdateFeed.checkForNew(false,0);
    },

    httpReadyStateChange: function() {
	if (hxUpdateFeed.httpReq.readyState == 2) {
	    try {
		hxUpdateFeed.httpReq.status;
	    } catch (e) {
		// 404 ?
		hxUpdateFeed.httpReq.abort();
		hxUpdateFeed.checkForNew(false,0);
	    }
	}
    },

    httpLoaded: function(e) {
	dump("hxUpdateFeed.httpLoaded(): Entered httpLoaded\n");

	if (hxUpdateFeed.currentFeed) {
	    dump("hxUpdateFeed.httpLoaded(): Current Feed exists.\n");
	} else {
	    dump("hxUpdateFeed.httpLoaded(): Current Feed DOES NOT exists.\n");
	}

	var localFile;
	dump("hxUpdateFeed.httpLoaded(): Trying to intiailize file object for " + hxUpdateFeed.currentPath + "\n");
	try {
	    dump("hxUpdateFeed.httpLoaded(): Trying to intiailize file object for " + hxUpdateFeed.currentPath + "\n");
	    localFile = new File(hxUpdateFeed.currentPath);
	} catch (e) {
	    dump("hxUpdateFeed.httpLoaded(): There was a problem creating file object for " +  hxUpdateFeed.currentPath + " : " + e + "\n");
	    hxUpdateFeed.httpReq.abort();
	    hxUpdateFeed.checkForNew(false,0);
	}

	dump("hxUpdateFeed.httpLoaded(): Finished initializing " + hxUpdateFeed.currentPath + "\n");
	if (! localFile.exists()) {
	    try {
		dump("hxUpdateFeed.httpLoaded(): File " +  hxUpdateFeed.currentPath + " does not exist. Creating it.\n");
		localFile.create();
	    } catch (e) {
		dump("hxUpdateFeed.httpLoaded(): There was a problem creating file " +  hxUpdateFeed.currentPath + " : " + e + "\n");
		hxUpdateFeed.httpReq.abort();
		hxUpdateFeed.checkForNew(false,0);
	    }
	}
	dump("hxUpdateFeed.httpLoaded(): File " +  hxUpdateFeed.currentPath + " successfully created.\n");

	try {
	    dump("hxUpdateFeed.httpLoaded(): Trying to write downloaded feed to " +  hxUpdateFeed.currentPath + "\n");
	    if (localFile.open('w') == 0) {
		// Thanks to Matthew Wilson <matthew@mjwilson.demon.co.uk>
		// for this hack to serialize downloaded content into UTF-8
		// charset Boingboing, Davezilla, Defective Yeti etc. feeds
		// did not work without this
		var serializer = new XMLSerializer();
		var str = Components.classes["@mozilla.org/supports-string;1"].createInstance(Components.interfaces.nsISupportsString);
		str.data = '';
		var outputStream = {
		    write: function(buf,count) {
			str.data += buf.substring(0,count);
			return count;
		    }
		};
		var charset = hxUpdateFeed.httpReq.responseXML.characterSet;
		serializer.serializeToStream(hxUpdateFeed.httpReq.responseXML, outputStream, UTF8);
		var content = str.data;
		// var re = new RegExp("<.xml.+>");
		var re = new RegExp(charset);
		content = content.replace(re, UTF8);  // Replace the encoding tag

		localFile.write(content);
	    } else {
		dump("hxUpdateFeed.httpLoaded(): Unabled to write downloaded feed to " +  hxUpdateFeed.currentPath + "\n");
	    }
	} catch (e) {
	    dump("hxUpdateFeed.httpLoaded(): Error writing downloaded data to " +  hxUpdateFeed.currentPath + ": " + e + "\n");
	}
	localFile.close();

	hxUpdateFeed.parseIntoHTML(hxUpdateFeed.currentPath, hxUpdateFeed.currentHTMLPath);
	hxUpdateFeed.checkForNew(true,hxUpdateFeed.newLastModified);
    },

    parseIntoHTML: function(aLocalPath,aLocalHTMLPath) {
	var feed = '';
	var parseError = 'Feed type is not recogonized as RSS, RDF, XML or Atom';

	try {
	    dump("hxUpdateFeed.parseIntoHTML(): parsing " + aLocalPath + "\n");
	    feed = new FeedParser('file://' + aLocalPath);

	    localHTMLPath = aLocalPath + ".html";
	    dump("localHTMLPath is " + localHTMLPath + "\n");
	} catch (e) {
	    parseError = e;
	    dump("hxUpdateFeed.parseIntoHTML(): Error parsing feed: " + e + "\n");
	}

	var parsedContent = '';
	var openInNew;
	var target = (getPrefValue("extensions.habarixenu.open_in_new", "bool", false)) ? " target='_blank'" : "";

	if (feed.type) {
	    dump("hxUpdateFeed.parseIntoHTML(): feed is " + feed.type + "\n");

	    var titleText = convertCharCodeFrom(feed.channel.title, UTF8);
	    var linkText = convertCharCodeFrom(feed.channel.link, UTF8);
	    var descText = convertCharCodeFrom(feed.channel.description, UTF8);

	    // Create a last modified timestamp based on SHA1 sums of first two links
	    var timeStamp = (feed.channel.item[0].link && feed.channel.item[1].link) ? hex_sha1(feed.channel.item[0].link + '' + feed.channel.item[1].link) : '';

	    hxUpdateFeed.newLastModified = timeStamp;
	    dump("hxUpdateFeed.parseIntoHTML(): LastModified is " + timeStamp + "\n");

	    parsedContent = "<HTML><HEAD>\n"
		+ "<TITLE>" + titleText + "</TITLE>\n"
		+ "<link rel='stylesheet' type='text/css' href='chrome://habarixenu/skin/display.css'>\n";
	    if (userStylesheet) {
		var du = new DirUtils();
		var userChrome = du.getUserChromeDir();
		var userContentCssFile = new File(userChrome);
		userContentCssFile.append('userContent.css');

		if (userContentCssFile.exists) {
		    parsedContent += "<link rel='stylesheet' type='text/css' href='file://" + userContentCssFile.path + "'>\n";
		}
	    }
	    parsedContent += "<META HTTP-EQUIV='Content-Type' CONTENT='text/html; charset=utf-8'>\n"
		+ "<BASE HREF='" + linkText + "'>\n"
		+ "</HEAD>\n<BODY onload='return false;'>\n"
		+ "<div class='hx-header'>\n"
		+ "<b class='hx-header-title' id='hx-header-title'><a href='" + linkText + "'" + target + ">" + titleText + "</a></b><br>\n"
		+ descText + "<br>\n";

	    if (showDetail) {
		parsedContent += "<p>\n<div class='hx-details' style='display: inline'>\n"
		    + "<b>" + strbundle.GetStringFromName("details") + "</b>:<br>"
		    + "<i>" + strbundle.GetStringFromName("feedurl") + "</i>: <a href='" + fetchURL + "'" + target + ">" + fetchURL + "</a><br>\n"
		    + "<i>" + strbundle.GetStringFromName("feedtype") + "</i>: " + convertCharCodeFrom(feed.type, UTF8) + "<br>\n";
		    // + "<i>" + strbundle.GetStringFromName("feeddownloadedat") + "</i>:" + lv + "<br>\n";
		if (feed.channel.pubDate) {
		    parsedContent += "<i>" + strbundle.GetStringFromName("feedpublishedat") + "</i>:" + convertCharCodeFrom(feed.channel.pubDate, UTF8) + "<br>\n";
		}
		parsedContent += "<i>" + strbundle.GetStringFromName("localhtmlcopy") + "</i>: <a href='file://" + localHTMLPath + "'" + target + ">" + localHTMLPath + "</a><br>\n" + "</div>\n";
	    }
	    parsedContent += "</div>\n";

	    for (var i = 0; i < feed.channel.item.length; i++) {
		parsedContent += "<div class='hx-item'>\n";
		if (feed.channel.item[i].title) {
		    parsedContent += "<b><a class='hx-headline' href='" + convertCharCodeFrom(feed.channel.item[i].link, UTF8) + "'" + target + ">" +
		    convertCharCodeFrom(feed.channel.item[i].title, UTF8) + "</a></b><br>\n";
		} else {
		    parsedContent += "<b><a class='hx-headline' href='" + convertCharCodeFrom(feed.channel.item[i].link, UTF8) + "'" + target + ">" +
		    convertCharCodeFrom(feed.channel.item[i].link, UTF8) + "</a></b><br>\n";
		}

		if (feed.channel.item[i].description) {
		    parsedContent += "<div class='hx-summary'>" + convertCharCodeFrom(feed.channel.item[i].description, UTF8) + "</div>\n";
		}

		// If we know category/author/date, then display it
		if (feed.channel.item[i].author || feed.channel.item[i].category || feed.channel.item[i].pubDate) {
		    parsedContent += "<br clear='all'>\n"
			+ "<div class='hx-credit'>Posted ";
		    if (feed.channel.item[i].author) {
			if (feed.channel.item[i].author.indexOf('by ') || feed.channel.item[i].author.indexOf('By ') || feed.channel.item[i].author.indexOf('BY ')) {
			    parsedContent += convertCharCodeFrom(feed.channel.item[i].author, UTF8) + " ";
			} else {
			    parsedContent += "by " + convertCharCodeFrom(feed.channel.item[i].author, UTF8) + " ";
			}
		    }

		    if (feed.channel.item[i].category) {
			parsedContent += "to section '" + convertCharCodeFrom(feed.channel.item[i].category, UTF8) + "' ";
		    }

		    if (feed.channel.item[i].pubDate) {
			parsedContent += "at " + convertCharCodeFrom(feed.channel.item[i].pubDate, UTF8) + " ";
		    }
		    parsedContent += "</div>\n";
		}
		parsedContent += "</div>\n";
	    }

	    if (feed.channel.copyright) {
		parsedContent += "<div align='center'>" + convertCharCodeFrom(feed.channel.copyright, UTF8) + "</div>";
	    }
	} else {
	    parsedContent = "<HTML><HEAD>\n"
		+ "<TITLE>" + strbundle.GetStringFromName("feedtypenotsupported") + "</TITLE>\n"
		+ "<link rel='stylesheet' type='text/css' href='chrome://habarixenu/skin/display.css'>\n"
		+ "</HEAD><BODY onload='return true;'>\n"
		+ "<div class='header'>\n"
		+ "<b class='header-title' id='header-title'>" + strbundle.GetStringFromName("feedtypenotsupported") + "</b><br>\n"
		+ strbundle.GetStringFromName("feedtypenotsupportedinfo")
		+ "<p><i>" + parseError + "</i><p>\n";
	    if (showDetail) {
		parsedContent += "<p><div class='details' style='display: inline'>\n"
		+ "<b>" + strbundle.GetStringFromName("details") + "</b>:<br>"
		+ "<i>" + strbundle.GetStringFromName("feedurl") + "</i>: <a href='" + fetchURL + "' target='_blank'>" + fetchURL + "</a><br>\n"
		+ "<i>" + strbundle.GetStringFromName("feedtype") + "</i>: Unknown<br>\n"
		// + "<i>" + strbundle.GetStringFromName("feeddownloadedat") + "</i>:" + lastUpdatedAt + "<br>\n"
		+ "</div>\n";
	    }
	    parsedContent += "</div>\n";
	}
	parsedContent += "</BODY>\n</HTML>";


	var feedHTMLFile;
	try {
	    feedHTMLFile = new File(localHTMLPath);
	} catch (e) {
	    dump("hxUpdateFeed.parseIntoHTML(): Error initializing file " + localHTMLPath + " : " + e + "\n");
	}

	if (! feedHTMLFile.exists()) {
	    dump(localHTMLPath + " does not exist.  Creating.\n");
	    dump("hxUpdateFeed.parseIntoHTML(): File " + localHTMLPath + " does not exist. Creating.\n");
	    feedHTMLFile.create();
	}

	if(feedHTMLFile.open('w') == 0) {
	    dump("hxUpdateFeed.parseIntoHTML(): Writing to file " + localHTMLPath + ".\n");
	    feedHTMLFile.write(parsedContent);
	} else {
	    dump("hxUpdateFeed.parseIntoHTML(): Can't open file " + localHTMLPath + " for writing.\n");
	}
	feedHTMLFile.close();
	hxUpdateFeed.setLVDate();

	var fileURLToFeedHTMLFile = 'file://' + localHTMLPath;
	dump("hxUpdateFeed.parseIntoHTML(): Wrote parsed content to " + fileURLToFeedHTMLFile + "\n");

	if (hxUpdateFeed.isClick) {
	    dump("hxUpdateFeed.parseIntoHTML(): This was through user click.\n");
	    openNewsInTab(fileURLToFeedHTMLFile);
	}

	return true;
    },

    checkForNew: function(aSuccess, aNewTimeStamp) {
	dump("hxUpdateFeed.checkForNew(): Called with:" + aSuccess + "," + aNewTimeStamp + "\n");
	// currentDescription
	// Parse out the parts of the current description
	var currentStatus = hxUpdateFeed.currentDescription.split(" -- ")[0];
	var currentTimeStamp = hxUpdateFeed.currentDescription.split(" -- ")[1];
	var newStatus;
	var newTimeStamp = "" + aNewTimeStamp;
	var newDescription;

	dump("hxUpdateFeed.checkForNew(): status:" + currentStatus + " timestamp: " + currentTimeStamp + "\n");
	dump("hxUpdateFeed.checkForNew(): newstatus:" + newStatus + " newtimestamp: " + newTimeStamp + "\n");

	if (aSuccess) {
	    if (hxUpdateFeed.isClick) {
		newDescription = "hxold -- " + newTimeStamp;
	    } else {
		if (currentStatus && currentTimeStamp) {
		    if (newTimeStamp == currentTimeStamp) {
			newDescription = "hxold -- " + newTimeStamp;
		    } else {
			newDescription = "hxnew -- " + newTimeStamp;
		    }
		} else {
		    // Nothing exists before. Assume the new one is the one
		    newDescription = "hxnew -- " + newTimeStamp;
		}
	    }
	}

	if (newDescription) {
	    dump("hxUpdateFeed.checkForNew(): Setting description to " + newDescription + "\n");
	    const descName = RDF.GetResource(NC_NS + "Description");
	    var rSource;
	    if (typeof(hxUpdateFeed.currentFeed) == "string") {
		rSource = RDF.GetResource(hxUpdateFeed.currentFeed);
	    } else {
		rSource = hxUpdateFeed.currentFeed;
	    }
	    var node =  BMDS.GetTarget(rSource, descName, true);

	    var newNode = RDF.GetLiteral(newDescription);

	    if (node) {
		dump("hxUpdateFeed.checkForNew(): An existing description. Changing it.\n");
		// A description exist, change it
		try {
		    BMDS.Change(rSource, descName, node, newNode);
		    BMDS.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
		} catch (e) {
		    dump("hxUpdateFeed.checkForNew(): Error changing description of bookmark: " + hxUpdateFeed.currentURL + " " + e + "\n");
		}
	    } else {
		dump("hxUpdateFeed.checkForNew(): No descrition existing. Creating one.\n");
		// A description does not exist. Create it
		try {
		    BMDS.Assert(rSource, descName, newNode, true);
		    BMDS.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
		} catch (e) {
		    dump("hxUpdateFeed.checkForNew(): Error asserting description of bookmark: " + hxUpdateFeed.currentURL + " " + e + "\n");
		}
	    }

	}

	if (this.downloadList.length == 0) {
	    this.checking = false;
	    this.onChecked(this.currentName, this.currentURL);
	    return;
	} else {
	    dump("hxUpdateFeed.checkForNew(): More feeds exist. Moving to next.\n");
	    this.downloadNext();
	}
    },

    setLVDate: function() {
	dump("hxUpdateFeed.setLVDate(): This is a newly downloaded feed.  Setting LastVisitDate in bookmarks.\n");

	var currentTime = new Date();
	var currentmicroseconds = currentTime.valueOf() * 1000;
	var newNode = RDF.GetDateLiteral(currentmicroseconds);

	const rName = RDF.GetResource(WEB_NS + "LastVisitDate");
	var rSource = RDF.GetResource(this.currentFeed);
	var node =  BMDS.GetTarget(rSource, rName, true);

	if (node) {
	    // A LastVisitDate exists for this bookmark.  Change it
	    try {
		// dump("Called with " + rSource + "," + rName + "," + node + "," + newNode + "\n");
		BMDS.Change(rSource, rName, node, newNode);
		BMDS.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
	    } catch (e) {
		dump("fetchFeed(): Error changing LastVisitDate of bookmark: " + e + "\n");
	    }
	} else {
	    // A LastVisitDate does not exist for this bookmark.  Create it
	    try {
		// dump("Called with " + rSource + "," + rName + "," + newNode + "\n");
		BMDS.Assert(rSource, rName, newNode, true);
		BMDS.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
	    } catch (e) {
		dump("fetchFeed(): Error changing LastVisitDate of bookmark: " + e + "\n");
	    }
	}
    },

    onCheck: function(aName,aURL) {},
    onChecked: function(aName,aURL) {}
}
